package com.bpcl.lpg.core.service.impl;

import de.hybris.platform.servicelayer.exceptions.SystemException;
import org.apache.logging.log4j.LogManager;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.fasterxml.jackson.databind.ObjectMapper;
import java.time.format.DateTimeFormatter;
import java.time.LocalDateTime;
import com.bpcl.lpg.facades.bda.data.LpgBdaReturnEmptyRequestData;
import com.bpcl.lpg.facades.bda.data.LpgBdaReturnEmptyCylinderResponseData;
import com.bpcl.core.model.SAPB2BUnitModel;
import de.hybris.platform.b2b.model.B2BUnitModel;
import com.bpcl.lpg.facades.bda.data.BdaAuthenticationRequestData;
import com.bpcl.lpg.facades.bda.data.BdaAuthenticationResponseData;
import java.util.Optional;
import com.bpcl.lpg.core.model.BDADeliveryChallanModel;
import java.util.function.Consumer;
import com.bpcl.lpg.facades.bda.data.LpgBdaAcknowledgeRequestData;
import com.bpcl.lpg.facades.bda.data.LpgAcknowledgeResponseData;
import com.bpcl.lpg.facades.bda.data.LpgBdaCreateIndentRequestData;
import com.bpcl.lpg.facades.bda.data.LpgCreateIndentResponseData;
import org.apache.commons.lang.StringUtils;
import com.bpcl.lpg.facades.bda.data.LpgBdaRequestData;
import com.bpcl.lpg.facades.bda.data.LpgBdaStocksRequestData;
import com.bpcl.lpg.facades.bda.data.LpgBdaStockResponseData;
import de.hybris.platform.servicelayer.exceptions.AmbiguousIdentifierException;
import de.hybris.platform.servicelayer.exceptions.ModelNotFoundException;
import java.util.Objects;
import com.bpcl.lpg.core.model.LpgDeliveryAssistantModel;
import com.bpcl.lpg.facades.data.LpgBDADeliveryConfirmResponseData;
import com.bpcl.lpg.facades.data.LpgBDADeliveryConfirmRequestData;
import com.fasterxml.jackson.core.JsonProcessingException;
import java.net.http.HttpResponse;
import java.text.MessageFormat;
import com.bpcl.lpg.core.pojo.RspResponseData;
import com.bpcl.lpg.facades.safetyInspection.data.CustomerDetailsRequestData;
import com.bpcl.lpg.core.service.BpclHttpClientService;
import com.bpcl.lpg.core.dao.LpgBDADao;
import com.bpcl.core.model.BPCLCustomerModel;
import com.bpcl.core.model.BPCLB2BUnitModel;
import de.hybris.platform.b2b.services.B2BUnitService;
import de.hybris.platform.servicelayer.model.ModelService;
import de.hybris.platform.servicelayer.keygenerator.KeyGenerator;
import com.bpcl.lpg.core.credentials.LPGCredentialsService;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import javax.annotation.Resource;
import com.bpcl.lpg.core.encryption.LpgEncryptionService;
import org.apache.logging.log4j.Logger;
import com.bpcl.lpg.core.service.LpgBDAService;

public class LpgBDAServiceImpl implements LpgBDAService
{
    private static final Logger LOG;
    private static final String BDA_CUSTOMER_DETAILS_API_NAME = "bda.customer.details.name";
    private static final String CORRELATION_HEADER_KEY = "X-BPCL-Correlation-Id";
    private static final String DEBUG_JSON_FORMATTER = "BDA Request Data to LPGNext :: {0}";
    private static final String PRODUCT_CODE_BPC142 = "BPC142";
    private static final String BDA_CUSTOMER_DETAILS_API_URL = "bda.customer.details.url";
    private static final String B2BUNIT_CODE_PREFIX = "0000";
    private static final String CREATE_INDENT_API_NAME = "bdaCreateIndentUrl";
    private static final String ACKNOWLEDGE_API_NAME = "bdaAcknowledgeFullUrl";
    private static final String CONFIRM_DELIVERY_API_NAME = "cashMemoGenerationAndDeliveryByBdaURL";
    private static final String RETURN_EMPTY_API_NAME = "bdaReturnEmptyCylinderUrl";
    private static final String STOCK_DETAILS_API_NAME = "bdaStockDetailsUrl";
    private static final String AUTHENTICATE_API_NAME = "bdaAuthenticateApi";
    @Resource
    private LpgEncryptionService lpgEncryptionService;
    @Resource(name = "configurationService")
    private ConfigurationService configurationService;
    @Resource
    private LPGCredentialsService lpgCredentialsService;
    @Resource
    private KeyGenerator bdaUniqueApiKeyGenerator;
    @Resource
    private KeyGenerator bdaDeliveryUniqueApiKeyGenerator;
    @Resource
    private KeyGenerator bdaCreateIndentUniqueApiKeyGenerator;
    @Resource
    private KeyGenerator bdaAuthUniqueApiKeyGenerator;
    @Resource(name = "modelService")
    private ModelService modelService;
    @Resource
    private B2BUnitService<BPCLB2BUnitModel, BPCLCustomerModel> b2bUnitService;
    @Resource
    private KeyGenerator bdaAcknowledgeUniqueApiKeyGenerator;
    @Resource
    private KeyGenerator bdaStocksUniqueApiKeyGenerator;
    @Resource
    private LpgBDADao lpgBDADao;
    @Resource
    private BpclHttpClientService bpclHttpClientService;
    @Resource
    private KeyGenerator bdaReturnEmptyCylinderKeyGenerator;
    
    public void populateRequestData(final CustomerDetailsRequestData requestData) {
        final String uniqueNumber = this.bdaUniqueApiKeyGenerator.generate().toString();
        requestData.setUniqueAPIReqNo(uniqueNumber);
        requestData.setPartnerTransId(uniqueNumber);
        final String hashValue = this.getHashKeyValueRequest(requestData);
        requestData.setHashKey(hashValue);
        this.getDefaultValuesForNonMandatoryFields(requestData);
    }
    
    public RspResponseData getCustomerDetailsFromLpg(final CustomerDetailsRequestData requestData) throws JsonProcessingException {
        this.populateRequestData(requestData);
        final String customerDetailsRequestUrl = MessageFormat.format(this.configurationService.getConfiguration().getString("bda.customer.details.url"), requestData.getUniqueAPIReqNo());
        final HttpResponse<String> httpResponse = this.bpclHttpClientService.executePostMethodForBDA(customerDetailsRequestUrl, this.getAsJsonBody(requestData), this.configurationService.getConfiguration().getString("bda.customer.details.name"));
        return this.convertToResponse(RspResponseData.class, httpResponse.body());
    }
    
    public LpgBDADeliveryConfirmResponseData getDeliveryConfirmationFromLpg(final LpgBDADeliveryConfirmRequestData requestData) throws JsonProcessingException {
        requestData.setEndPointName("cashMemoGenerationAndDeliveryByBdaURL");
        this.populateDeliveryConfirmationRequestData(requestData);
        final String getBDADeliveryConfirmationUrl = MessageFormat.format(this.configurationService.getConfiguration().getString("cashMemoGenerationAndDeliveryByBdaURL"), requestData.getUniqueAPIReqNo());
        final HttpResponse<String> httpResponse = this.bpclHttpClientService.executePostMethodForBDA(getBDADeliveryConfirmationUrl, this.getAsJsonBody(requestData), "cashMemoGenerationAndDeliveryByBdaURL");
        return this.convertToResponse(LpgBDADeliveryConfirmResponseData.class, httpResponse.body());
    }
    
    public LpgDeliveryAssistantModel getBDADetailsByUid(final String userId) {
        return this.lpgBDADao.getBDADetailsByUid(userId);
    }
    
    public boolean doesBdaExist(final String contactNumber) {
        try {
            final LpgDeliveryAssistantModel deliveryAssistantModel = this.getBDADetailsByUid(contactNumber);
            return Objects.nonNull(deliveryAssistantModel);
        }
        catch (final ModelNotFoundException | AmbiguousIdentifierException exception) {
            return false;
        }
    }
    
    public void populateDeliveryConfirmationRequestData(final LpgBDADeliveryConfirmRequestData requestData) {
        final String uniqueNumber = this.bdaDeliveryUniqueApiKeyGenerator.generate().toString();
        requestData.setUniqueAPIReqNo(uniqueNumber);
        requestData.setPartnerTransId(uniqueNumber);
        final String hashValue = this.getDeliveryHashKeyValue(requestData);
        requestData.setHashValue(hashValue);
        this.getDefaultValuesForDeliveryConfirmation(requestData);
    }
    
    public LpgBdaStockResponseData getBDAStockDetails(final String mobileNumber) throws JsonProcessingException {
        final LpgDeliveryAssistantModel lpgDeliveryAssistantModel = this.getBDADetailsByUid(mobileNumber);
        final LpgBdaStocksRequestData lpgBdaStocksRequestData = new LpgBdaStocksRequestData();
        final String uniqueNumber = this.bdaStocksUniqueApiKeyGenerator.generate().toString();
        this.populateBdaRequestData((LpgBdaRequestData)lpgBdaStocksRequestData, lpgDeliveryAssistantModel, "bdaStockDetailsUrl", uniqueNumber);
        lpgBdaStocksRequestData.setHashValue(this.getHashKeyValueRequest(lpgBdaStocksRequestData));
        final String lpgNextStockDetailsApiRequestUrl = MessageFormat.format(this.configurationService.getConfiguration().getString("bdaStockDetailsUrl"), uniqueNumber);
        final HttpResponse<String> httpResponse = this.bpclHttpClientService.executePostMethodForBDA(lpgNextStockDetailsApiRequestUrl, this.getAsJsonBody(lpgBdaStocksRequestData), "bdaStockDetailsUrl");
        return this.convertToResponse(LpgBdaStockResponseData.class, httpResponse.body());
    }
    
    private void populateBdaRequestData(final LpgBdaRequestData lpgBdaRequestData, final LpgDeliveryAssistantModel lpgDeliveryAssistant, final String endPointName, final String uniqueNumber) {
        final String partnerCode = this.lpgCredentialsService.getPartnerCodeByApiName(endPointName);
        lpgBdaRequestData.setPartnerCode(partnerCode);
        lpgBdaRequestData.setPartnerTransId(uniqueNumber);
        lpgBdaRequestData.setUniqueAPIReqNo(uniqueNumber);
        lpgBdaRequestData.setDeliveryAssistantCode(lpgDeliveryAssistant.getDeliveryAssistantCode());
        lpgBdaRequestData.setDeliveryAssistantMobileNo(lpgDeliveryAssistant.getMobileNo());
        lpgBdaRequestData.setDistCode(lpgDeliveryAssistant.getDistributorId());
        lpgBdaRequestData.setAddl1("");
        lpgBdaRequestData.setAddl2("");
        lpgBdaRequestData.setAddl3("");
        lpgBdaRequestData.setAddl4("");
        lpgBdaRequestData.setAddl5("");
    }
    
    private String getHashKeyValueRequest(final LpgBdaStocksRequestData lpgBdaStocksRequestData) {
        final StringBuilder hashMessage = new StringBuilder();
        final String secret = this.lpgCredentialsService.getPartnerSecretByApiName("bdaStockDetailsUrl");
        hashMessage.append(lpgBdaStocksRequestData.getPartnerCode());
        hashMessage.append(lpgBdaStocksRequestData.getPartnerTransId());
        hashMessage.append(lpgBdaStocksRequestData.getUniqueAPIReqNo());
        hashMessage.append(this.getEmptyIfBlank(lpgBdaStocksRequestData.getDeliveryAssistantCode()));
        hashMessage.append(this.getEmptyIfBlank(lpgBdaStocksRequestData.getDeliveryAssistantMobileNo()));
        hashMessage.append(this.getEmptyIfBlank(lpgBdaStocksRequestData.getDistCode()));
        hashMessage.append(this.getEmptyIfBlank(secret));
        return this.lpgEncryptionService.encodeBySHA512(hashMessage.toString());
    }
    
    private void getDefaultValuesForNonMandatoryFields(final CustomerDetailsRequestData data) {
        if (StringUtils.isBlank(data.getPartnerTransId())) {
            data.setPartnerTransId("");
        }
        if (StringUtils.isBlank(data.getEndPointName())) {
            data.setEndPointName("");
        }
        if (StringUtils.isBlank(data.getAddl1())) {
            data.setAddl1("");
        }
        if (StringUtils.isBlank(data.getAddl2())) {
            data.setAddl2("");
        }
        if (StringUtils.isBlank(data.getAddl3())) {
            data.setAddl3("");
        }
        if (StringUtils.isBlank(data.getConsumerNo())) {
            data.setConsumerNo("");
        }
        if (StringUtils.isBlank(data.getLpgId())) {
            data.setLpgId("");
        }
        if (StringUtils.isBlank(data.getDistCode())) {
            data.setDistCode("");
        }
        if (StringUtils.isBlank(data.getContactNumber())) {
            data.setContactNumber("");
        }
        if (StringUtils.isBlank(data.getContactNumberType())) {
            data.setContactNumberType("");
        }
    }
    
    private void getDefaultValuesForDeliveryConfirmation(final LpgBDADeliveryConfirmRequestData data) {
        data.setAddl1("");
        data.setAddl2("");
        data.setAddl3("");
        data.setAddl4("");
        data.setAddl5("");
        data.setDeliveryMode("");
        data.setPaymentType("");
        data.setContactNumberType("");
        if (StringUtils.isBlank(data.getConsumerNo())) {
            data.setConsumerNo("");
        }
        if (StringUtils.isBlank(data.getLpgId())) {
            data.setLpgId("");
        }
        if (StringUtils.isBlank(data.getDistCode())) {
            data.setDistCode("");
        }
        if (StringUtils.isBlank(data.getConsumerContactNumber())) {
            data.setConsumerContactNumber("");
        }
    }
    
    private String getHashKeyValueRequest(final CustomerDetailsRequestData requestData) {
        final StringBuilder hashMessage = new StringBuilder();
        final String endPointName = this.configurationService.getConfiguration().getString("bda.customer.details.name");
        final String partnerCode = this.lpgCredentialsService.getPartnerCodeByApiName(endPointName);
        final String secret = this.lpgCredentialsService.getPartnerSecretByApiName(endPointName);
        requestData.setPartnerCode(partnerCode);
        hashMessage.append(partnerCode);
        hashMessage.append(requestData.getPartnerTransId());
        hashMessage.append(requestData.getUniqueAPIReqNo());
        hashMessage.append(this.getEmptyIfBlank(requestData.getContactNumberType()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getContactNumber()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getLpgId()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getDistCode()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getConsumerNo()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getAddl1()));
        hashMessage.append(secret);
        final String encryptedMsg = this.lpgEncryptionService.encodeBySHA512(hashMessage.toString());
        return encryptedMsg;
    }
    
    private String getDeliveryHashKeyValue(final LpgBDADeliveryConfirmRequestData requestData) {
        final StringBuilder hashMessage = new StringBuilder();
        final String endPointName = requestData.getEndPointName();
        final String partnerCode = this.lpgCredentialsService.getPartnerCodeByApiName(endPointName);
        final String secret = this.lpgCredentialsService.getPartnerSecretByApiName(endPointName);
        requestData.setPartnerCode(partnerCode);
        hashMessage.append(partnerCode);
        hashMessage.append(requestData.getPartnerTransId());
        hashMessage.append(requestData.getUniqueAPIReqNo());
        hashMessage.append(this.getEmptyIfBlank(requestData.getContactNumberType()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getConsumerContactNumber()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getLpgId()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getDistCode()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getConsumerNo()));
        hashMessage.append(requestData.getDeliveryAssistantCode());
        hashMessage.append(requestData.getDeliveryAssistantMobileNo());
        hashMessage.append(secret);
        final String encryptedMsg = this.lpgEncryptionService.encodeBySHA512(hashMessage.toString());
        return encryptedMsg;
    }
    
    public LpgCreateIndentResponseData createIndent(final String productCode, final String quantity, final LpgDeliveryAssistantModel lpgDeliveryAssistant) throws JsonProcessingException {
        final LpgBdaCreateIndentRequestData createIndentRequest = new LpgBdaCreateIndentRequestData();
        final String uniqueNumber = this.bdaCreateIndentUniqueApiKeyGenerator.generate().toString();
        this.populateBdaRequestData((LpgBdaRequestData)createIndentRequest, lpgDeliveryAssistant, "bdaCreateIndentUrl", uniqueNumber);
        createIndentRequest.setProductCode(productCode);
        createIndentRequest.setRequestQuantity(quantity);
        createIndentRequest.setCreatedBy(lpgDeliveryAssistant.getDeliveryAssistantCode());
        createIndentRequest.setHashValue(this.getCreateIndentHashValue(createIndentRequest));
        final String lpgNextCreateIndentApiRequestUrl = MessageFormat.format(this.configurationService.getConfiguration().getString("bdaCreateIndentUrl"), uniqueNumber);
        final HttpResponse<String> httpResponse = this.bpclHttpClientService.executePostMethodForBDA(lpgNextCreateIndentApiRequestUrl, this.getAsJsonBody(createIndentRequest), "bdaCreateIndentUrl");
        return this.convertToResponse(LpgCreateIndentResponseData.class, httpResponse.body());
    }
    
    private String getCreateIndentHashValue(final LpgBdaCreateIndentRequestData createIndentRequest) {
        final StringBuilder hashMessage = new StringBuilder();
        final String secret = this.lpgCredentialsService.getPartnerSecretByApiName("bdaCreateIndentUrl");
        hashMessage.append(createIndentRequest.getPartnerCode());
        hashMessage.append(createIndentRequest.getPartnerTransId());
        hashMessage.append(createIndentRequest.getUniqueAPIReqNo());
        hashMessage.append(this.getEmptyIfBlank(createIndentRequest.getDeliveryAssistantCode()));
        hashMessage.append(this.getEmptyIfBlank(createIndentRequest.getDeliveryAssistantMobileNo()));
        hashMessage.append(this.getEmptyIfBlank(createIndentRequest.getDistCode()));
        hashMessage.append(this.getEmptyIfBlank(createIndentRequest.getProductCode()));
        hashMessage.append(this.getEmptyIfBlank(createIndentRequest.getRequestQuantity()));
        hashMessage.append(this.getEmptyIfBlank(createIndentRequest.getCreatedBy()));
        hashMessage.append(this.getEmptyIfBlank(secret));
        return this.lpgEncryptionService.encodeBySHA512(hashMessage.toString());
    }
    
    public LpgAcknowledgeResponseData acknowledgeDeliveryChallan(final String deliveryChallanNo, final String ackStatus) throws JsonProcessingException {
        final BDADeliveryChallanModel deliveryChallanModel = this.getBdaDeliveryChallan(deliveryChallanNo);
        final LpgDeliveryAssistantModel lpgDeliveryAssistant = this.getBDADetailsByUid("bda".concat(deliveryChallanModel.getDeliveryAssistantMobileNo()));
        final String uniqueNumber = this.bdaAcknowledgeUniqueApiKeyGenerator.generate().toString();
        final LpgBdaAcknowledgeRequestData acknowledgeRequest = new LpgBdaAcknowledgeRequestData();
        this.populateBdaRequestData((LpgBdaRequestData)acknowledgeRequest, lpgDeliveryAssistant, "bdaAcknowledgeFullUrl", uniqueNumber);
        acknowledgeRequest.setIndentRequestId(deliveryChallanModel.getIndentRequestId());
        acknowledgeRequest.setDeliveryChallanNo(deliveryChallanNo);
        acknowledgeRequest.setAckStatus(ackStatus);
        acknowledgeRequest.setAckBy(lpgDeliveryAssistant.getDeliveryAssistantCode());
        acknowledgeRequest.setHashValue(this.getAcknowledgeDeliveryChallanHashValue(acknowledgeRequest));
        final String lpgNextAckDeliveryChallanRequestUrl = MessageFormat.format(this.configurationService.getConfiguration().getString("bdaAcknowledgeFullUrl"), uniqueNumber);
        final HttpResponse<String> httpResponse = this.bpclHttpClientService.executePostMethodForBDA(lpgNextAckDeliveryChallanRequestUrl, this.getAsJsonBody(acknowledgeRequest), "bdaAcknowledgeFullUrl");
        final LpgAcknowledgeResponseData responseData = this.convertToResponse(LpgAcknowledgeResponseData.class, httpResponse.body());
        if (null != httpResponse.headers()) {
            final Optional<String> firstValue;
            final Optional<String> correlation = firstValue = httpResponse.headers().firstValue("X-BPCL-Correlation-Id");
            final BDADeliveryChallanModel obj = deliveryChallanModel;
            Objects.requireNonNull(obj);
            firstValue.ifPresent(obj::setCorrelationId);
        }
        if ("200".equals(responseData.getReturnCode()) || "201".equals(responseData.getReturnCode()) || "717".equals(responseData.getReturnCode())) {
            deliveryChallanModel.setStatus("Accept");
        }
        else if ("202".equals(responseData.getReturnCode()) || "203".equals(responseData.getReturnCode()) || "718".equals(responseData.getReturnCode()) || "720".equals(responseData.getReturnCode())) {
            deliveryChallanModel.setStatus("Reject");
        }
        deliveryChallanModel.setReturnCode(responseData.getReturnCode());
        deliveryChallanModel.setReturnMsg(responseData.getReturnMsg());
        this.modelService.save((Object)deliveryChallanModel);
        return responseData;
    }
    
    private String getAcknowledgeDeliveryChallanHashValue(final LpgBdaAcknowledgeRequestData acknowledgeRequest) {
        final StringBuilder hashMessage = new StringBuilder();
        final String secret = this.lpgCredentialsService.getPartnerSecretByApiName("bdaAcknowledgeFullUrl");
        hashMessage.append(acknowledgeRequest.getPartnerCode());
        hashMessage.append(acknowledgeRequest.getPartnerTransId());
        hashMessage.append(acknowledgeRequest.getUniqueAPIReqNo());
        hashMessage.append(this.getEmptyIfBlank(acknowledgeRequest.getDeliveryAssistantCode()));
        hashMessage.append(this.getEmptyIfBlank(acknowledgeRequest.getDeliveryAssistantMobileNo()));
        hashMessage.append(this.getEmptyIfBlank(acknowledgeRequest.getDistCode()));
        hashMessage.append(this.getEmptyIfBlank(acknowledgeRequest.getIndentRequestId()));
        hashMessage.append(this.getEmptyIfBlank(acknowledgeRequest.getDeliveryChallanNo()));
        hashMessage.append(this.getEmptyIfBlank(acknowledgeRequest.getAckStatus()));
        hashMessage.append(this.getEmptyIfBlank(acknowledgeRequest.getAckBy()));
        hashMessage.append(this.getEmptyIfBlank(secret));
        return this.lpgEncryptionService.encodeBySHA512(hashMessage.toString());
    }
    
    public BdaAuthenticationResponseData getBDADetailsFromLpg(final String contactNumber) throws JsonProcessingException {
        final BdaAuthenticationRequestData bdaRequestData = new BdaAuthenticationRequestData();
        this.populateBDARequestData(contactNumber, bdaRequestData);
        final String getBDAAuthenticationUrl = MessageFormat.format(this.configurationService.getConfiguration().getString("bdaAuthenticateApi"), bdaRequestData.getUniqueAPIReqNo());
        final HttpResponse<String> httpResponse = this.bpclHttpClientService.executePostMethodForBDA(getBDAAuthenticationUrl, this.getAsJsonBody(bdaRequestData), "bdaAuthenticateApi");
        return this.convertToResponse(BdaAuthenticationResponseData.class, httpResponse.body());
    }
    
    private void populateBDARequestData(final String contactNumber, final BdaAuthenticationRequestData bdaRequestData) {
        final String uniqueNumber = this.bdaAuthUniqueApiKeyGenerator.generate().toString();
        bdaRequestData.setPartnerTransId(uniqueNumber);
        bdaRequestData.setUniqueAPIReqNo(uniqueNumber);
        bdaRequestData.setContactNumber(contactNumber);
        bdaRequestData.setEndPoint("bdaAuthenticateApi");
        final String hashValue = this.getAuthHashKeyValueRequest(bdaRequestData);
        bdaRequestData.setHashValue(hashValue);
        this.updateNonMandatoryFieldsAsEmpty(bdaRequestData);
    }
    
    private String getAuthHashKeyValueRequest(final BdaAuthenticationRequestData requestData) {
        final StringBuilder hashMessage = new StringBuilder();
        final String endPointName = requestData.getEndPoint();
        final String partnerCode = this.lpgCredentialsService.getPartnerCodeByApiName(endPointName);
        final String secret = this.lpgCredentialsService.getPartnerSecretByApiName(endPointName);
        requestData.setPartnerCode(partnerCode);
        hashMessage.append(partnerCode);
        hashMessage.append(requestData.getPartnerTransId());
        hashMessage.append(requestData.getUniqueAPIReqNo());
        hashMessage.append(this.getEmptyIfBlank(requestData.getContactNumber()));
        hashMessage.append(secret);
        final String encryptedMsg = this.lpgEncryptionService.encodeBySHA512(hashMessage.toString());
        return encryptedMsg;
    }
    
    private void updateNonMandatoryFieldsAsEmpty(final BdaAuthenticationRequestData requestData) {
        requestData.setAddl1("");
        requestData.setAddl2("");
        requestData.setAddl3("");
    }
    
    public void createDeliveryAssistant(final BdaAuthenticationResponseData bdaResponseData, final String customerId, final Boolean isBdaUpdate, LpgDeliveryAssistantModel bdaModel) {
        if (!isBdaUpdate) {
            bdaModel = (LpgDeliveryAssistantModel)this.modelService.create((Class)LpgDeliveryAssistantModel.class);
            bdaModel.setUid(customerId);
            bdaModel.setDefaultB2BUnit((B2BUnitModel)this.b2bUnitService.getUnitForUid("Dummy"));
            bdaModel.setEmail("dummy@test.com");
        }
        if (Objects.nonNull(bdaResponseData.getData())) {
            if (StringUtils.isNotBlank(bdaResponseData.getData().getDeliveryAssistantCode())) {
                bdaModel.setDeliveryAssistantCode(bdaResponseData.getData().getDeliveryAssistantCode());
            }
            if (StringUtils.isNotBlank(bdaResponseData.getData().getDeliveryAssistantName())) {
                bdaModel.setDeliveryAssistantName(bdaResponseData.getData().getDeliveryAssistantName());
            }
            if (StringUtils.isNotBlank(bdaResponseData.getData().getDeliveryAssistantType())) {
                bdaModel.setDeliveryAssistantType(bdaResponseData.getData().getDeliveryAssistantType());
            }
            if (StringUtils.isNotBlank(bdaResponseData.getData().getMobileNo())) {
                bdaModel.setMobileNo(bdaResponseData.getData().getMobileNo());
            }
            if (StringUtils.isNotBlank(bdaResponseData.getData().getVehicleCapacity())) {
                bdaModel.setVehicleCapacity(Integer.valueOf(bdaResponseData.getData().getVehicleCapacity()));
            }
            if (StringUtils.isNotBlank(bdaResponseData.getData().getVehicleType())) {
                bdaModel.setVehicleType(bdaResponseData.getData().getVehicleType());
            }
            if (StringUtils.isNotBlank(bdaResponseData.getData().getDistributorCode())) {
                bdaModel.setDistributorId(bdaResponseData.getData().getDistributorCode());
                final SAPB2BUnitModel b2bunit = this.lpgBDADao.getDistributorNameByCode("0000" + bdaResponseData.getData().getDistributorCode());
                if (Objects.nonNull(b2bunit)) {
                    bdaModel.setDistributorName(b2bunit.getName());
                }
            }
        }
        this.modelService.save((Object)bdaModel);
    }
    
    public LpgBdaReturnEmptyCylinderResponseData createReturnRequestForCylinder(final String vehicleNumber, final String requestedQuantity, final String mobileNumber) throws JsonProcessingException {
        final LpgDeliveryAssistantModel lpgDeliveryAssistant = this.getBDADetailsByUid(mobileNumber);
        final LpgBdaReturnEmptyRequestData returnEmptyRequestData = new LpgBdaReturnEmptyRequestData();
        final String uniqueNumber = this.bdaReturnEmptyCylinderKeyGenerator.generate().toString();
        this.populateBdaRequestData((LpgBdaRequestData)returnEmptyRequestData, lpgDeliveryAssistant, "bdaReturnEmptyCylinderUrl", uniqueNumber);
        final LocalDateTime requestedDate = LocalDateTime.now();
        final DateTimeFormatter timeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss.SSS");
        final String formattedRequestedDate = requestedDate.format(timeFormatter);
        returnEmptyRequestData.setProductCode("BPC142");
        returnEmptyRequestData.setRequestQuantity(requestedQuantity);
        returnEmptyRequestData.setVehicleNumber(vehicleNumber);
        returnEmptyRequestData.setRequestDate(formattedRequestedDate);
        returnEmptyRequestData.setCreatedBy(lpgDeliveryAssistant.getDeliveryAssistantCode());
        returnEmptyRequestData.setHashValue(this.getReturnEmptyCylinderHashValue(returnEmptyRequestData));
        final String lpgNextReturnRequestUrl = MessageFormat.format(this.configurationService.getConfiguration().getString("bdaReturnEmptyCylinderUrl"), uniqueNumber);
        final HttpResponse<String> httpResponse = this.bpclHttpClientService.executePostMethodForBDA(lpgNextReturnRequestUrl, this.getAsJsonBody(returnEmptyRequestData), "bdaReturnEmptyCylinderUrl");
        return this.convertToResponse(LpgBdaReturnEmptyCylinderResponseData.class, httpResponse.body());
    }
    
    private String getReturnEmptyCylinderHashValue(final LpgBdaReturnEmptyRequestData returnEmptyRequestData) {
        final StringBuilder hashMessage = new StringBuilder();
        final String secret = this.lpgCredentialsService.getPartnerSecretByApiName("bdaReturnEmptyCylinderUrl");
        hashMessage.append(returnEmptyRequestData.getPartnerCode());
        hashMessage.append(returnEmptyRequestData.getPartnerTransId());
        hashMessage.append(returnEmptyRequestData.getUniqueAPIReqNo());
        hashMessage.append(this.getEmptyIfBlank(returnEmptyRequestData.getDeliveryAssistantCode()));
        hashMessage.append(this.getEmptyIfBlank(returnEmptyRequestData.getDeliveryAssistantMobileNo()));
        hashMessage.append(this.getEmptyIfBlank(returnEmptyRequestData.getDistCode()));
        hashMessage.append(this.getEmptyIfBlank(returnEmptyRequestData.getProductCode()));
        hashMessage.append(this.getEmptyIfBlank(returnEmptyRequestData.getRequestQuantity()));
        hashMessage.append(this.getEmptyIfBlank(returnEmptyRequestData.getVehicleNumber()));
        hashMessage.append(this.getEmptyIfBlank(returnEmptyRequestData.getRequestDate()));
        hashMessage.append(this.getEmptyIfBlank(returnEmptyRequestData.getCreatedBy()));
        hashMessage.append(this.getEmptyIfBlank(secret));
        return this.lpgEncryptionService.encodeBySHA512(hashMessage.toString());
    }
    
    private <T> T convertToResponse(final Class<T> responseObject, final String responseBody) throws JsonProcessingException {
        final ObjectMapper mapper = new ObjectMapper();
        mapper.setVisibility(PropertyAccessor.ALL, JsonAutoDetect.Visibility.NONE);
        mapper.setVisibility(PropertyAccessor.FIELD, JsonAutoDetect.Visibility.ANY);
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        mapper.configure(DeserializationFeature.ACCEPT_EMPTY_STRING_AS_NULL_OBJECT, true);
        return mapper.readValue(responseBody, responseObject);
    }
    
    private String getAsJsonBody(final Object obj) throws JsonProcessingException {
        final ObjectMapper objectMapper = new ObjectMapper();
        final String json = objectMapper.writerWithDefaultPrettyPrinter().writeValueAsString(obj);
        if (LpgBDAServiceImpl.LOG.isDebugEnabled()) {
            LpgBDAServiceImpl.LOG.debug(MessageFormat.format("BDA Request Data to LPGNext :: {0}", json));
        }
        return objectMapper.writeValueAsString(obj);
    }
    
    private String getEmptyIfBlank(final String stringToCheck) {
        return StringUtils.isBlank(stringToCheck) ? "" : stringToCheck;
    }
    
    public BDADeliveryChallanModel getBdaDeliveryChallan(final String deliveryChallanNo) {
        return this.lpgBDADao.getBdaDeliveryChallan(deliveryChallanNo);
    }
    
    public BDADeliveryChallanModel getBdaDeliveryChallanByMobileNo(final String mobileNumber) {
        return this.lpgBDADao.getBdaDeliveryChallanByMobileNo(mobileNumber);
    }
    
    static {
        LOG = LogManager.getLogger((Class)LpgBDAServiceImpl.class);
    }
}
